import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:webview_flutter/webview_flutter.dart';

import '../cidaas_login_provider.dart';
import '../entity/cidaas_config.dart';
import '../entity/token_entity.dart';
import './../authentification/authentication_bloc.dart';

class LoginBrowser extends StatefulWidget {
  final String? reRouteToAfterLogin;
  final Widget splashScreen;

  LoginBrowser({Key? key, this.reRouteToAfterLogin, required this.splashScreen})
      : super(key: key);

  @override
  _LoginBrowserState createState() =>
      _LoginBrowserState(this.reRouteToAfterLogin, this.splashScreen);
}

class _LoginBrowserState extends State<LoginBrowser> {
  late AuthenticationBloc _authenticationBloc;
  String _reRouteToAfterLogin = '';
  late Widget _splashScreen;

  _LoginBrowserState(String? reRouteToAfterLogin, Widget splashScreen) {
    if (reRouteToAfterLogin != null) {
      this._reRouteToAfterLogin = reRouteToAfterLogin;
    }
    this._splashScreen = splashScreen;
  }

  @override
  void initState() {
    super.initState();
    _authenticationBloc = BlocProvider.of<AuthenticationBloc>(context);
  }

  final Key comKey = const Key('login');

  @override
  Widget build(BuildContext context) {
    Widget getWebView() {
      final Future<String> _initURL = CidaasLoginProvider.getLoginURL();
      return FutureBuilder<String>(
          future: _initURL, // a previously-obtained Future<String> or null
          builder: (BuildContext context, AsyncSnapshot<String> urlSnapshot) {
            if (urlSnapshot.hasData) {
              final Future<CidaasConfig> _config =
                  CidaasLoginProvider.getCidaasConf();
              return FutureBuilder<CidaasConfig>(
                  future: _config,
                  // a previously-obtained Future<String> or null
                  builder: (BuildContext context,
                      AsyncSnapshot<CidaasConfig> configSnapshot) {
                    if (configSnapshot.hasData) {
                      //addListener(configSnapshot.data!);
                      final CidaasConfig config = configSnapshot.data!;
                      return WebViewWidget(
                          controller: WebViewController()
                            ..setJavaScriptMode(JavaScriptMode.unrestricted)
                            ..setBackgroundColor(const Color(0x00000000))
                            ..setNavigationDelegate(
                              NavigationDelegate(
                                onNavigationRequest:
                                    (NavigationRequest request) async {
                                  if (request.url
                                      .startsWith(config.redirectURI)) {
                                    final Uri parsedUrl =
                                        Uri.parse(request.url);
                                    final String? code =
                                        parsedUrl.queryParameters['code'];
                                    final TokenEntity? tokenEntity =
                                        await CidaasLoginProvider
                                            .getAccessTokenByCode(
                                                code.toString());
                                    if (tokenEntity != null) {
                                      _authenticationBloc.add(
                                          AuthenticationLoggedInEvent(
                                              tokenEntity: tokenEntity));
                                      Navigator.of(context)
                                          .pushReplacementNamed(
                                              this._reRouteToAfterLogin,
                                              arguments: tokenEntity);
                                      return NavigationDecision.prevent;
                                    }
                                  }
                                  return NavigationDecision.navigate;
                                },
                              ),
                            )
                            ..loadRequest(
                                Uri.parse(urlSnapshot.data as String)));
                    } else if (configSnapshot.hasError) {
                      // The config needs to be set by the developer
                      throw configSnapshot.error!;
                    } else {
                      return this._splashScreen;
                    }
                  });
            } else if (urlSnapshot.hasError) {
              // The url needs to be set in the config by the developer
              throw urlSnapshot.error!;
            } else {
              return this._splashScreen;
            }
          });
    }

    return getWebView();
  }
}
